import GenGisAPIv1 as ggAPI
import dataHelper as dh
from datetime import date
from datetime import timedelta
import time

def dateStrToDate(dateStr):
	"""
	Convert date stored in a string as month, day, year (i.e. 12/1/2009) to a date object 
	"""
	from datetime import date
	
	dateSplit = dateStr.split('/')
	month = int(dateSplit[0])
	day = int(dateSplit[1])
	year = int(dateSplit[2])
	
	return date(year, month, day)
	
def lessEqualDateMDY(dateStr1, dateStr2):
	"""
	Compare two dates specified as strings in month, day, year order (i.e. 12/1/2009)
	
	@return: True if date1 <= date2, else False
	"""
	
	return dateStrToDate(dateStr1) <= dateStrToDate(dateStr2)
	
def SizeSetter(count):
	if count <= 1:
		return 3
	elif count < 10:
		return 5
	elif count < 100:
		return 7
	else:
		return 9

def ColourSetter(deltaDate):
	alpha = 1.0 - float(deltaDate.days)/28
	if alpha < 0.0:
		alpha = 0.0
		
	return  ggAPI.Colour(1.0, 0.0, 0.0, alpha)
	
def CreateLegend():
	sizes = [3, 5, 7, 9];
	labels = ["1 case", "< 10 cases", "< 100 cases", ">= 100 cases"]
	
	index = 0
	for size in sizes:	
		marker = ggAPI.VisualMarker(ggAPI.Colour(1.0, 0, 0), size, ggAPI.MARKER_SHAPE.CIRCLE, ggAPI.Point3D(40, 285-25*index, 0))
		marker.SetRenderingStyle(ggAPI.MARKER_RENDERING_STYLE.ORTHO)
		ggAPI.addMarker(marker)
		
		label = ggAPI.VisualLabel(labels[index], ggAPI.Colour(1.0, 1.0, 1.0), 18)
		label.SetScreenPosition(ggAPI.Point3D(52,285-25*index-6,1))
		label.SetRenderingStyle(ggAPI.LABEL_RENDERING_STYLE.ORTHO)
		ggAPI.addLabel(label)
		
		index = index + 1
			
def showSpread(startDate, endDate):
	# get all location sites
	locData = dh.getLocationSetData()
	
	# set camera position
	ggAPI.setYaw(0.0)
	ggAPI.setPitch(90.0)
	ggAPI.setPosition(0.0, 0.0, 1.5785574913024902)
	ggAPI.refresh()
	
	# set initial visual properties of all location sites
	for i in xrange(0, len(locData)):
		locData[i].SetColour(ggAPI.Colour(1.0, 0.5, 0))
		locData[i].SetSize(4.0)
		locData[i].SetVisibility(False)
		
	ggAPI.refresh()
	
	# label for showing current date
	label = ggAPI.VisualLabel("", ggAPI.Colour(0.0, 0.0, 0.0), 20)
	label.SetScreenPosition(ggAPI.Point3D(40.0, 420.0, 1.0))
	label.SetRenderingStyle(ggAPI.LABEL_RENDERING_STYLE.ORTHO)
	ggAPI.addLabel(label)
	
	# create legend
	CreateLegend()
	
	# show spread of virus on a daily basis	
	dayDelta = timedelta(days=1)
	
	# get cases from start date to current simulation date
	curDate = startDate
	while(curDate <= endDate):
		startTime = time.time()
		curDateStr = str(curDate.month) + '/' + str(curDate.day) + '/' + str(curDate.year)
		filteredLoc = dh.filterData(locData, 'Date', curDateStr, lessEqualDateMDY)
		
		# determine number of cases and last reported case for all locations
		numCases = {}
		lastReportedCase = {}
		uniqueLoc = []
		for loc in filteredLoc:
			loc.SetVisibility(False)
			if loc.id in numCases:
				numCases[loc.id] = numCases[loc.id] + 1
				locDate = dateStrToDate(loc.data['Date'])
				if lastReportedCase[loc.id] < locDate:
					lastReportedCase[loc.id] = locDate
			else:
				numCases[loc.id] = 1
				lastReportedCase[loc.id] = dateStrToDate(loc.data['Date'])
				uniqueLoc.append(loc)
				
		# set visual properties of location sites
		for loc in uniqueLoc:
			deltaDate = curDate - lastReportedCase[loc.id]
			loc.SetColour(ColourSetter(deltaDate))
			loc.SetRenderingOrder(deltaDate.days*10);
			loc.SetSize(SizeSetter(numCases[loc.id]))
			loc.SetVisibility(True)
		
		# set date of label
		label.SetText(str(curDate.day) + "/" + str(curDate.month) + "/" + str(curDate.year))
				
		ggAPI.refresh()
		ggAPI.wxSafeYield()
		
		curDate = curDate + dayDelta
		
		elapsedTime = time.time() - startTime
		if elapsedTime < 1.5:
			time.sleep(1.5 - elapsedTime)



